/* coin_acceptor PRom configuration file */

// Config values for PRom support
#ifdef HAVE_PROM_SUPPORT
# define CA_PROM_VERSION 		"2.04"		// PRom version (do not touch)

# define CA_PROM_ANSWER_LENGTH 		300		// Default length of a PRom answer
# define CA_PROM_REQUEST_LENGTH		10		// Size of a request including its header
# define CA_PROM_HEADCMP_LENGTH		8		// Size of header to compare against request

# define CA_PROM_HEADER_READ    	0x4f		// Header for reading a PRom value
# define CA_PROM_HEADER_WRITE   	0x4e		// Header for writing a PRom value

# define CA_PROM_HEADER_WRITE_OK       	0x5001		// Header for a write success answer

# define CA_PROM_LONG			0		// PRom write type long
# define CA_PROM_STRING			1		// PRom write type string
# define CA_PROM_STRING_MAXLENGTH	16		// PRom string value max length
#endif

// Config values from direct command support
#ifdef HAVE_COMMAND_SUPPORT
# define CA_CONVERT_BIN 	0		// Superglobal constant to determine the hex2bin function
# define CA_CONVERT_HEX 	1		// Superglobal constant to determine the bin2hex function
# define CA_CONVERT_LONG	2		// Superglobal constant to determine the hex2long function
#endif

// Global configuration values
#define CA_SLOT_A 		0	// Internal number of Slot A
#define CA_SLOT_B 		1	// Internal number of Slot B
#define CA_SLOT_C 		2	// Internal number of Slot C
#define CA_SLOT_D 		3	// Internal number of Slot D

#define CA_ANSWER_LENGTH 	32	// Default length of a Coin Acceptor answer
#define CA_RESET_REQUEST_LENGTH	4	// Default length of a reset request

// Coin Acceptor addressing
#define CA_PIT_A         0x410041	// Request value of Slot A
#define CA_PIT_B         0x420042	// Request value of Slot B
#define CA_PIT_C         0x430043	// Request value of Slot C
#define CA_PIT_D         0x440044	// Request value of Slot D

#define CA_PIT_A_OK      0x4101		// Answer header for an active A Slot
#define CA_PIT_B_OK      0x4201		// Answer header for an active B Slot
#define CA_PIT_C_OK      0x4301		// Answer header for an active C Slot
#define CA_PIT_D_OK      0x4401		// Answer header for an active D Slot

#define CA_REJECT        0x520052	// Reject coins, deny insertion
#define CA_FREE          0x460046	// Accept coins, permit insertion

#define CA_REJECT_OK     0x520052	// Valid answer to CA_REJECT
#define CA_FREE_OK       0x460046	// Valid answer to CA_FREE

// Programmable Read only memory addressing
#ifdef HAVE_PROM_SUPPORT
# define CA_PROM_STRING_0	"12120477"	// String field 0
# define CA_PROM_STRING_1	"12121487"	// String field 1
# define CA_PROM_STRING_2	"12122497"	// String field 2
# define CA_PROM_STRING_3	"121234a7"	// String field 3
# define CA_PROM_STRING_4	"121244b7"	// String field 4
# define CA_PROM_STRING_5	"121254c7"	// String field 5
# define CA_PROM_STRING_6	"121264d7"	// String field 6
# define CA_PROM_STRING_7	"121274e7"	// String field 7
# define CA_PROM_STRING_8	"121284f7"	// String field 8
# define CA_PROM_STRING_9	"16100075"	// String field 9
# define CA_PROM_STRING_10	"16101085"	// String field 10
# define CA_PROM_STRING_11	"16102095"	// String field 11
# define CA_PROM_STRING_12	"161030a5"	// String field 12
# define CA_PROM_STRING_13	"161040b5"	// String field 13
# define CA_PROM_STRING_14	"161050c5"	// String field 14
# define CA_PROM_STRING_15	"161060d5"	// String field 15
# define CA_PROM_STRING_16	"161070e5"	// String field 16
# define CA_PROM_STRING_17	"161080f5"	// String field 17
# define CA_PROM_STRING_18	"1b158605"	// String field 18
# define CA_PROM_STRING_19	"1b159615"	// String field 19
# define CA_PROM_STRING_20	"1b15a625"	// String field 20
# define CA_PROM_STRING_21	"1b15b635"	// String field 21
# define CA_PROM_STRING_22	"1b15d655"	// String field 22
# define CA_PROM_STRING_23	"1b15e665"	// String field 23
# define CA_PROM_STRING_24	"1b15f675"	// String field 24

# define CA_PROM_LONG_0		"06110066"	// Long field 0
# define CA_PROM_LONG_1		"06111076"	// Long field 1
# define CA_PROM_LONG_2		"06112086"	// Long field 2
# define CA_PROM_LONG_3		"06113096"	// Long field 3
# define CA_PROM_LONG_4		"061140a6"	// Long field 4
# define CA_PROM_LONG_5		"061150b6"	// Long field 5
# define CA_PROM_LONG_6		"061160c6"	// Long field 6
# define CA_PROM_LONG_7		"061170d6"	// Long field 7
# define CA_PROM_LONG_8		"061180e6"	// Long field 8
# define CA_PROM_LONG_9		"061190f6"	// Long field 9
# define CA_PROM_LONG_10	"03110568"	// Long field 10
# define CA_PROM_LONG_11	"03111578"	// Long field 11
# define CA_PROM_LONG_12	"03112588"	// Long field 12
# define CA_PROM_LONG_13	"03112588"	// Long field 13
# define CA_PROM_LONG_14	"03113598"	// Long field 14
# define CA_PROM_LONG_15	"031145a8"	// Long field 15
# define CA_PROM_LONG_16	"031155b8"	// Long field 16
# define CA_PROM_LONG_17	"031165c8"	// Long field 17
# define CA_PROM_LONG_18	"031175d8"	// Long field 18
# define CA_PROM_LONG_19	"031185e8"	// Long field 19
# define CA_PROM_LONG_20	"031195f8"	// Long field 20
#endif
