/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.0 of the PHP license,       |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_0.txt.                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Oliver Welter <areaz2@php.net>                               |
  +----------------------------------------------------------------------+
*/

/* $Id: header,v 1.15 2004/01/08 16:46:52 areaz2 Exp $ */

#ifdef PHP_WIN32
#error "This extension is only for Linux based systems"
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_coin_acceptor.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>

/* e.g. IRIX does not have CRTSCTS */
#ifndef CRTSCTS
# ifdef CNEW_RTSCTS
#  define CRTSCTS CNEW_RTSCTS
# else
#  define CRTSCTS 0
# endif /* CNEW_RTSCTS */
#endif /* !CRTSCTS */

#define le_coin_acceptor_name "Coin Acceptor Device Descriptor"

// The default delays
long CA_DELAY_COINS = 65535;

#ifdef HAVE_PROM_SUPPORT	
 long CA_DELAY_PROM = 65535;
#endif

ZEND_DECLARE_MODULE_GLOBALS(coin_acceptor)

/* True global resources - no need for thread safety here */
static int le_coin_acceptor;

/* {{{ coin_acceptor_functions[]
 *
 * Every user visible function must have an entry in coin_acceptor_functions[].
 */
function_entry coin_acceptor_functions[] = {
#ifdef HAVE_COMMAND_SUPPORT
	PHP_FE(ca_convert, NULL)
#endif
	PHP_FE(ca_open, NULL)
	PHP_FE(ca_init, NULL)
	PHP_FE(ca_read, NULL)
	PHP_FE(ca_reset, NULL)
	PHP_FE(ca_lock, NULL)
	PHP_FE(ca_unlock, NULL)
	PHP_FE(ca_accept_coins, NULL)
#ifdef HAVE_PROM_SUPPORT	
	PHP_FE(ca_prom_get, NULL)
	PHP_FE(ca_prom_set, NULL)
#endif
#ifdef HAVE_COMMAND_SUPPORT
	PHP_FE(ca_command, NULL)
#endif
	PHP_FE(ca_close, NULL)
	{NULL, NULL, NULL}	/* Must be the last line in coin_acceptor_functions[] */
};
/* }}} */

/* {{{ coin_acceptor_module_entry
 */
zend_module_entry coin_acceptor_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"coin_acceptor",
	coin_acceptor_functions,
	PHP_MINIT(coin_acceptor),
	PHP_MSHUTDOWN(coin_acceptor),
	NULL,
	NULL,
	PHP_MINFO(coin_acceptor),
#if ZEND_MODULE_API_NO >= 20010901
	"0.3", 
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_COIN_ACCEPTOR
ZEND_GET_MODULE(coin_acceptor)
#endif

static void _ca_close_fd(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
        php_fd_t *f = (php_fd_t *) rsrc->ptr;
        close(f->fd);
        free(f);
}

#define RCAC(c) REGISTER_LONG_CONSTANT(#c, c, CONST_CS | CONST_PERSISTENT)
#define RCASC(c) REGISTER_STRING_CONSTANT(#c, c, CONST_CS | CONST_PERSISTENT)
#define CA_UNDEF_CONST -1

/* {{{ PHP_INI
 */
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("coin_acceptor.auto_reset",	"On", PHP_INI_ALL, OnUpdateString, auto_reset, zend_coin_acceptor_globals, coin_acceptor_globals)
    STD_PHP_INI_ENTRY("coin_acceptor.auto_initialize",	"Off", PHP_INI_ALL, OnUpdateString, auto_initialize, zend_coin_acceptor_globals, coin_acceptor_globals)
    STD_PHP_INI_ENTRY("coin_acceptor.start_unlocked",	"On", PHP_INI_ALL, OnUpdateString, start_unlocked, zend_coin_acceptor_globals, coin_acceptor_globals)
    STD_PHP_INI_ENTRY("coin_acceptor.lock_on_close",	"Off", PHP_INI_ALL, OnUpdateString, lock_on_close, zend_coin_acceptor_globals, coin_acceptor_globals)
#ifdef HAVE_COMMAND_SUPPORT
    STD_PHP_INI_ENTRY("coin_acceptor.command_function",	"Off", PHP_INI_ALL, OnUpdateString, command_function, zend_coin_acceptor_globals, coin_acceptor_globals)
#endif    
    STD_PHP_INI_ENTRY("coin_acceptor.delay_coins",	"53132", PHP_INI_ALL, OnUpdateLong, delay_coins, zend_coin_acceptor_globals, coin_acceptor_globals)
#ifdef HAVE_PROM_SUPPORT	
    STD_PHP_INI_ENTRY("coin_acceptor.delay_prom",	"55748", PHP_INI_ALL, OnUpdateLong, delay_prom, zend_coin_acceptor_globals, coin_acceptor_globals)
#endif
PHP_INI_END()
/* }}} */

/* {{{ php_coin_acceptor_init_globals
 */
static void php_coin_acceptor_init_globals(zend_coin_acceptor_globals *coin_acceptor_globals)
{
	coin_acceptor_globals->auto_reset = NULL;
	coin_acceptor_globals->auto_initialize = NULL;
	coin_acceptor_globals->start_unlocked = NULL;
	coin_acceptor_globals->lock_on_close = NULL;
#ifdef HAVE_COMMAND_SUPPORT
	coin_acceptor_globals->command_function = NULL;
#endif
	coin_acceptor_globals->delay_coins = 0;
#ifdef HAVE_PROM_SUPPORT	
	coin_acceptor_globals->delay_prom = 0;
#endif
}
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(coin_acceptor)
{        
        le_coin_acceptor = zend_register_list_destructors_ex(_ca_close_fd, NULL, le_coin_acceptor_name, module_number);

	RCAC(CA_SLOT_A);
	RCAC(CA_SLOT_B);
	RCAC(CA_SLOT_C);
	RCAC(CA_SLOT_D);

#ifdef HAVE_PROM_SUPPORT	
	RCASC(CA_PROM_STRING_0);
	RCASC(CA_PROM_STRING_1);
	RCASC(CA_PROM_STRING_2);
	RCASC(CA_PROM_STRING_3);
	RCASC(CA_PROM_STRING_4);
	RCASC(CA_PROM_STRING_5);
	RCASC(CA_PROM_STRING_6);
	RCASC(CA_PROM_STRING_7);
	RCASC(CA_PROM_STRING_8);
	RCASC(CA_PROM_STRING_9);
	RCASC(CA_PROM_STRING_10);
	RCASC(CA_PROM_STRING_11);
	RCASC(CA_PROM_STRING_12);
	RCASC(CA_PROM_STRING_13);
	RCASC(CA_PROM_STRING_14);
	RCASC(CA_PROM_STRING_15);
	RCASC(CA_PROM_STRING_16);
	RCASC(CA_PROM_STRING_17);
	RCASC(CA_PROM_STRING_18);
	RCASC(CA_PROM_STRING_19);
	RCASC(CA_PROM_STRING_20);
	RCASC(CA_PROM_STRING_21);
	RCASC(CA_PROM_STRING_22);
	RCASC(CA_PROM_STRING_23);
	RCASC(CA_PROM_STRING_24);

	RCASC(CA_PROM_LONG_0);
	RCASC(CA_PROM_LONG_1);
	RCASC(CA_PROM_LONG_2);
	RCASC(CA_PROM_LONG_3);
	RCASC(CA_PROM_LONG_4);
	RCASC(CA_PROM_LONG_5);
	RCASC(CA_PROM_LONG_6);
	RCASC(CA_PROM_LONG_7);
	RCASC(CA_PROM_LONG_8);
	RCASC(CA_PROM_LONG_9);
	RCASC(CA_PROM_LONG_10);
	RCASC(CA_PROM_LONG_11);
	RCASC(CA_PROM_LONG_12);
	RCASC(CA_PROM_LONG_13);
	RCASC(CA_PROM_LONG_14);
	RCASC(CA_PROM_LONG_15);
	RCASC(CA_PROM_LONG_16);
	RCASC(CA_PROM_LONG_17);
	RCASC(CA_PROM_LONG_18);
	RCASC(CA_PROM_LONG_19);
	RCASC(CA_PROM_LONG_20);
#endif
	
	RCAC(CA_ANSWER_LENGTH);
#ifdef HAVE_PROM_SUPPORT	
	RCAC(CA_PROM_ANSWER_LENGTH);

	RCAC(CA_CONVERT_BIN);
	RCAC(CA_CONVERT_HEX);
	RCAC(CA_CONVERT_LONG);
#endif
	
	ZEND_INIT_MODULE_GLOBALS(coin_acceptor, php_coin_acceptor_init_globals, NULL);
	REGISTER_INI_ENTRIES();
	
	CA_DELAY_COINS = COIN_ACCEPTOR_G(delay_coins);
#ifdef HAVE_PROM_SUPPORT
	CA_DELAY_PROM = COIN_ACCEPTOR_G(delay_prom);

#endif
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(coin_acceptor)
{
	UNREGISTER_INI_ENTRIES();

	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(coin_acceptor)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "coin_acceptor support", "enabled");
#ifdef HAVE_PROM_SUPPORT
        php_info_print_table_header(2, "coin_acceptor prom support", "enabled");
        php_info_print_table_header(2, "coin_acceptor prom version", CA_PROM_VERSION);
#endif
#ifdef HAVE_COMMAND_SUPPORT
	php_info_print_table_header(2, "direct command support", "enabled");
#endif
	php_info_print_table_end();

	DISPLAY_INI_ENTRIES();
}
/* }}} */

static void new_php_ca_fd(php_fd_t **f, int fd)
{
        *f = malloc(sizeof(php_fd_t));
        (*f)->fd = fd;

        (*f)->accept_coins = 0;
        (*f)->ca_initialized = 0;
}

static void update_php_ca_fd(php_fd_t **f, php_fd_t *ca)
{
        *f = malloc(sizeof(php_fd_t));
        (*f)->fd = ca->fd;
        
        (*f)->accept_coins = ca->accept_coins;
        (*f)->ca_initialized = ca->ca_initialized;
}

static void ca_sleep(long delay)
{
        // Give the coin acceptor / prom some time to process a request

        if ((delay > 25000) && (delay < 1000000))
        {
          usleep(delay);
        } else {
          // This is the default value      
          usleep(65535);
        }
}

/* A PHP3 rip out */
static char hexconvtab[] = "0123456789abcdef";
static char *bin2hex( const unsigned char *old, const size_t oldlen )
{
  unsigned char *new = NULL;
  size_t i, j;

  new = (char *) malloc(oldlen * 2 * sizeof(char));
  if(!new) {
    return new;
  }

  for(i = j = 0; i < oldlen; i++) {
    new[j++] = hexconvtab[old[i] >> 4];
    new[j++] = hexconvtab[old[i] & 15];
  }
  new[j] = '\0';
  return new;
}

int is_hex(char *given_chain, int len)
{
  int i;
  for (i = 0; i < len; i++)
    if (isxdigit(given_chain[i]) == 0)
      return 0;
  return 1;                                 
  return 1;
}

/* Convert a string to long */
static unsigned long hex2long(char *s, char **n) 
{

	unsigned long result = 0L;

	int digit;


	/* ignore leading whitespace */ 
	while (*s == ' ' || *s == '\t')

		s++;


	/* NB. only works for lower case letters */ 
	for (;;) {

		assert(!(*s >= 'A' && *s <= 'Z'));	/* lower case only */

		if (*s >= '0' && *s <= '9')

			digit = *s - '0';

		else if (*s >= 'a' && *s <= 'f')

			digit = *s - 'a' + 10;

		else

			break;

		result = (result << 4) | digit;

		s++;

	}							/* for */

	if (n != NULL)

		*n = s;

	return result;

}

/*  Converts a string of (an even number of) hex digits to binary */
char *hex2bin(unsigned char *hex, int len)
{
  char ch ;
  unsigned char val;
  unsigned char *tmpchain ;
  int nbytes = 0, i = 0, upper = 1 ;
  
  /* The chain should have 2*n characters */
  if (len % 2 != 0)
      return NULL ;
  if (!is_hex(hex, len))
      return NULL ;

  tmpchain = malloc((len / 2) + 1);

  for(; nbytes<len; hex++) {
    ch = *hex;
    if(ch == ' ') continue;
    if(islower(ch)) ch = (char)toupper(ch);
    if(isdigit(ch)) {
      val = (unsigned char) (ch - '0');
    } else if(ch>='A' && ch<='F') {
      val = (unsigned char)(ch - 'A' + 10);

      /* End of hex digits--time to bail out. */
    } else {
      return (upper ? tmpchain : 0);
    }

    /* If this is an upper digit, set the top 4 bits of the destination
     * byte with this value, else -OR- in the value.
     */
    if(upper) {              
      tmpchain[i] = (unsigned char) (val << 4);
      upper = 0;
    } else {
      tmpchain[i++] |= val;
      upper = 1;
    }
  }
  tmpchain[(len / 2)] = '\0' ;
  return(tmpchain);
}

int ca_write(int fd, unsigned char *request)
{
        int result;
        
	result = write(fd, hex2bin(request, strlen(request)), (strlen(request) / 2));
	
	if (result <= 0)
	{
          return -1;
        }

  	return result;
}

int ca_read(int fd, unsigned char *answer, int bytes)
{
        int result;
        
        result = read(fd, answer, bytes);
        
        if (result <= 0)
        {
          return -1;
        }
        
        return result;
}

int ca_hexcmp(unsigned char *result, unsigned char *hexval)
{
        unsigned char *hexresult;
        int cmplen = 2;
        int retval = 0;
        int resultlen, hexvallen;
        
        hexresult = bin2hex(result, strlen(result));
        
        resultlen = strlen(hexresult);
        hexvallen = strlen(hexval);
        
        if (resultlen < hexvallen)
        {
          cmplen = resultlen;
        } else {
          cmplen = hexvallen;
        }
        retval = strncmp(hexresult, hexval, cmplen);

        return retval;
}

int answer_to_lower(char *string, char *destination)
{
    int length = strlen(string);
    int temp;
    int tmp;
    int i;

    for (i = 0; i < length; i++) {
      temp = (int)string[i];
      tmp = tolower(temp);
      destination[i] = (char)tmp;
    }

    return 0;
}

int ca_reset_slot(php_fd_t *f, int slot)
{
        unsigned char 	answer[CA_ANSWER_LENGTH];
        unsigned char	request[CA_ANSWER_LENGTH];
        unsigned char 	resetreq[CA_RESET_REQUEST_LENGTH];
        unsigned char	result[CA_ANSWER_LENGTH];
        unsigned char 	*rreq;
	char	 	*pit[4];
	int	 	i;

	pit[CA_SLOT_A] = "A";
	pit[CA_SLOT_B] = "B";
	pit[CA_SLOT_C] = "C";
	pit[CA_SLOT_D] = "D";

	        switch (slot)
	        {
	          case CA_SLOT_A:
	            sprintf(request,"%x", CA_PIT_A);
	            break;
                  case CA_SLOT_B:
                    sprintf(request,"%x", CA_PIT_B);
                    break;
                  case CA_SLOT_C:
                    sprintf(request,"%x", CA_PIT_C);
                    break;
                  case CA_SLOT_D:
                    sprintf(request,"%x", CA_PIT_D);
                    break;
	        }

	        ca_write(f->fd, request);
	        ca_sleep(CA_DELAY_COINS);
	        if (ca_read(f->fd, answer, CA_ANSWER_LENGTH) <= 0)
	        {
	          return -1;
	        }
	        
	        switch (slot)
	        {
	          case CA_SLOT_A:
	            sprintf(result, "%x", CA_PIT_A_OK);
	            break;
                  case CA_SLOT_B:
	            sprintf(result, "%x", CA_PIT_B_OK);
                    break;
                  case CA_SLOT_C:
	            sprintf(result, "%x", CA_PIT_C_OK);
                    break;
                  case CA_SLOT_D:
	            sprintf(result, "%x", CA_PIT_D_OK);
                    break;
	        }
	        
	        if (ca_hexcmp(answer, result) != 0)
	        {
	          return 1;
	        }
	        

	answer_to_lower(answer, resetreq);
	rreq = bin2hex(resetreq,CA_RESET_REQUEST_LENGTH);
  
        ca_write(f->fd, rreq);
        ca_sleep(CA_DELAY_COINS);
        ca_read(f->fd, answer, CA_ANSWER_LENGTH);

	if (ca_hexcmp(answer, rreq) == 0)
	{
	  return 0;
	} else {
	  return 4;
	}
}

int ca_initialize(php_fd_t *f, int unlocked)
{
        unsigned char   *answer;
        unsigned char	request[CA_ANSWER_LENGTH];
        unsigned char	result[CA_ANSWER_LENGTH];
	int	 	i;
        ssize_t   	res;

	for (i = 0; i < 4; i++)
	{
	        switch (i)
	        {
	          case 0:
	          case 1:
  	            sprintf(request, "%x", CA_PIT_A);
  	            break;
	          case 2:
  	            sprintf(request, "%x", CA_REJECT);
  	            break;
	          case 3:
	            if (unlocked == 1) 
	            {
	              sprintf(request, "%x", CA_FREE);
	              f->accept_coins = 1;
	              update_php_ca_fd(&f, f);
                    }
  	            break;
	        }
	        ca_write(f->fd, request);
	        ca_sleep(CA_DELAY_COINS);
	        answer = emalloc(CA_ANSWER_LENGTH + 1);
	        res = ca_read(f->fd, answer, CA_ANSWER_LENGTH);

	        if (res <= 0)
	        {
		  efree(answer);
		  return 1;
		}

		answer = erealloc(answer, res + 1);
		answer[res] = 0;
	        
	        switch (i)
	        {
	          case 0:
                  case 1: // Test default answer twice
                    sprintf(result,"%x",CA_PIT_A_OK);
                    break;
                  case 2: // Reject all coins
                    sprintf(result,"%x",CA_REJECT_OK);
                    break;
                  case 3: // Free coin acceptor
                    if (unlocked == 1)
                    {
                      sprintf(result,"%x",CA_FREE_OK);
                    }
                    break;
	        }

	        if (ca_hexcmp(answer, result) != 0)
	        {
	          efree(answer);
	          return 2;
	        }
	}

        efree(answer);	
	return 0;
}

#ifdef HAVE_COMMAND_SUPPORT
/* {{{ proto string ca_convert(string arg, long conversion)
   Return a hex or character converted string */
PHP_FUNCTION(ca_convert)
{
	char 	*arg;
	int 	arg_len;
	int 	str_len;
	long 	mode;
 
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &arg, &arg_len, &mode) == FAILURE) {
		return;
	}

         unsigned char *string;
	
	switch (mode)
	{
		case CA_CONVERT_BIN:
                        if (!is_hex(arg, arg_len))
                        {
                          php_error_docref(NULL TSRMLS_CC, E_WARNING, "not a hex string: %s", string);
                          RETURN_FALSE;
                        }        
		        string = hex2bin(arg, strlen(arg));
			break;
		case CA_CONVERT_HEX:
		        string = bin2hex(arg, strlen(arg));
			break;
                case CA_CONVERT_LONG:
                        if (!is_hex(arg, arg_len))
                        {
                          php_error_docref(NULL TSRMLS_CC, E_WARNING, "not a hex string: %s", string);
                          RETURN_FALSE;
                        }        
                        int mval = hex2long(arg, NULL);
                        sprintf(string, "%d", mval);
                        break;
	}

	RETURN_STRINGL(string, strlen(string), 1);
}
/* }}} */
#endif

/* {{{ proto resource ca_open([string device])
   Open a coin acceptor device */
PHP_FUNCTION(ca_open)
{
        php_fd_t 	*f;
        char     	*ca_device;
        int       	ca_device_length;
        int       	fd;
        int	  	start_unlocked = 0;
        int	  	io;
        struct termios 	ti;

        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &ca_device, &ca_device_length) == FAILURE) {
                return;
        }

        if (ca_device) {
                fd = open(ca_device, O_RDWR);
        } else {
                php_error_docref(NULL TSRMLS_CC, E_WARNING, "ca_open() expects parameter 1 to be the device name");
                RETURN_FALSE;
        }

        if (fd == -1) {
                php_error_docref(NULL TSRMLS_CC, E_WARNING, "cannot open device %s: %s", ca_device, strerror(errno));
                RETURN_FALSE;
        }
        
        // Port setup
        if (ioctl(fd, TCGETS, &ti) < 0)
        {
                php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to setup device %s: %s", ca_device, strerror(errno));
                RETURN_FALSE;          
        } else {
          cfsetispeed(&ti, B9600);
          cfsetospeed(&ti, B9600);
          
          ti.c_lflag = IEXTEN;
          ti.c_oflag = ONOCR;
         
          ioctl(fd, TCSETS, &ti);
        }

        if ((COIN_ACCEPTOR_G(start_unlocked) == "On")
        || (COIN_ACCEPTOR_G(start_unlocked) == "on"))
        {
          start_unlocked = 1;
        }
        
        new_php_ca_fd(&f, fd);
        
        if ((COIN_ACCEPTOR_G(auto_initialize) == "On")
        || (COIN_ACCEPTOR_G(auto_initialize) == "on"))
        {
          if (ca_initialize(f, start_unlocked) != 0)
          {
                  php_error_docref(NULL TSRMLS_CC, E_WARNING, "cannot initialize device %s", ca_device);
                  RETURN_FALSE;
          } else {
	    f->ca_initialized = 1;          
          }
        }
        
        update_php_ca_fd(&f, f);

        ZEND_REGISTER_RESOURCE(return_value, f, le_coin_acceptor);
}

/* {{{ proto bool ca_init(resource fd)
   Initialize a coin acceptor by fd descriptor */
PHP_FUNCTION(ca_init)
{
        zval     	*r_fd;
        php_fd_t 	*f;
        int 		start_unlocked = 0;
        int 		retval;

        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &r_fd) == FAILURE) {
                return;
        }

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);

        if ((COIN_ACCEPTOR_G(start_unlocked) == "On")
	|| (COIN_ACCEPTOR_G(start_unlocked) == "on"))
	{
	  start_unlocked = 1;
	}
	
	switch(ca_initialize(f, start_unlocked))
	{
	  case 0:
	    f->ca_initialized = 1;
	    update_php_ca_fd(&f, f);
	    
	    RETURN_TRUE;
	    break;
          case 1:
            php_error_docref(NULL TSRMLS_CC, E_WARNING, "no answer from device");
            RETURN_FALSE;
            break;
          case 2:
            php_error_docref(NULL TSRMLS_CC, E_WARNING, "device has not answered as expected");
            RETURN_FALSE;
            break;
	}	
}

#define ADD_LFIELD(f, v) add_assoc_long_ex(return_value, (f), sizeof(f), v);
#define ADD_CFIELD(f, v) add_assoc_string_ex(return_value, (f), sizeof(f), v, 1);

/* {{{ proto array ca_read(resource fd)
   Read from coin acceptor by fd descriptor */
PHP_FUNCTION(ca_read)
{
        zval     	*r_fd;
        php_fd_t 	*f;
        unsigned char	*answer[CA_ANSWER_LENGTH];
        unsigned char	request[CA_ANSWER_LENGTH];
        unsigned char	result[CA_ANSWER_LENGTH];
	char	 	*pit[4];
	int	 	i;
        ssize_t		res;

        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &r_fd) == FAILURE) {
                return;
        }

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);
        
        if (f->ca_initialized == 0)
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "device not intialized");
          RETURN_FALSE;            
        }

	pit[CA_SLOT_A] = "A";
	pit[CA_SLOT_B] = "B";
	pit[CA_SLOT_C] = "C";
	pit[CA_SLOT_D] = "D";
	
	for (i = 0; i < 4; i++)
	{
	        answer[i] = emalloc(CA_ANSWER_LENGTH + 1);

	        switch (i)
	        {
	          case CA_SLOT_A:
	            sprintf(request,"%x",CA_PIT_A);
	            break;
                  case CA_SLOT_B:
                    sprintf(request,"%x",CA_PIT_B);
                    break;
                  case CA_SLOT_C:
                    sprintf(request,"%x",CA_PIT_C);
                    break;
                  case CA_SLOT_D:
                    sprintf(request,"%x",CA_PIT_D);
                    break;
	        }
	        
	        ca_write(f->fd, request);
                ca_sleep(CA_DELAY_COINS);
	        res = ca_read(f->fd, answer[i], CA_ANSWER_LENGTH);

	        if (res <= 0)
	        {
		  efree(answer[i]);
                  php_error_docref(NULL TSRMLS_CC, E_WARNING, "no answer from device, while reading slot %s", pit[i]);
                  RETURN_FALSE;       
		}

		answer[i] = erealloc(answer[i], res + 1);
		answer[i][res] = 0;
	        
	        switch (i)
	        {
	          case CA_SLOT_A:
	            sprintf(result,"%x",CA_PIT_A_OK);
	            break;
                  case CA_SLOT_B:
	            sprintf(result,"%x",CA_PIT_B_OK);
                    break;
                  case CA_SLOT_C:
	            sprintf(result,"%x",CA_PIT_C_OK);
                    break;
                  case CA_SLOT_D:
	            sprintf(result,"%x",CA_PIT_D_OK);
                    break;
	        }

	        if (ca_hexcmp(answer[i], result) != 0)
	        {
	            php_error_docref(NULL TSRMLS_CC, E_WARNING, "device has not answered as expected, while reading slot %s", pit[i]);
	            RETURN_FALSE;       
	        }
	}

	int coins[4];
	unsigned char chex[1];
	unsigned char *hexval;
	int a = 0;
	long b;
	
	for (i = 0; i < 4; i++)
	{ 
            coins[i] = 0;
            sprintf(chex,"%c",answer[i][2]);

            hexval = bin2hex(chex,1);
            a = hex2long(hexval,NULL);
            
            if ((a < 0) || (a > 256))
            {
	      php_error_docref(NULL TSRMLS_CC, E_WARNING, "slot %s has stored more than 255 coins, please reset first", pit[i]);
	      RETURN_FALSE;       
            } else {
              coins[i] = a;
            }
            
            // Autoreset, if configured
            if ((COIN_ACCEPTOR_G(auto_reset) == "on")
            || (COIN_ACCEPTOR_G(auto_reset) == "On"))
            {
              switch (i)
              {
                case CA_SLOT_A:
                  if (coins[CA_SLOT_A] > 0) ca_reset_slot(f, CA_SLOT_A);
                  break;
                case CA_SLOT_B:
                  if (coins[CA_SLOT_B] > 0) ca_reset_slot(f, CA_SLOT_B);
                  break;
                case CA_SLOT_C:
                  if (coins[CA_SLOT_C] > 0) ca_reset_slot(f, CA_SLOT_C);
                  break;
                case CA_SLOT_D:
                  if (coins[CA_SLOT_D] > 0) ca_reset_slot(f, CA_SLOT_D);
                  break;
              }
            }
            efree(answer[i]);
	}
	
        // Struct return array
        array_init(return_value);
        ADD_LFIELD("slot_a", coins[CA_SLOT_A]);
        ADD_LFIELD("slot_b", coins[CA_SLOT_B]);
        ADD_LFIELD("slot_c", coins[CA_SLOT_C]);
        ADD_LFIELD("slot_d", coins[CA_SLOT_D]);
}

/* {{{ proto bool ca_reset(resource fd, int slot)
   Reset slot given by fd */
PHP_FUNCTION(ca_reset)
{
        zval     *r_fd;
        php_fd_t *f;
        int	 slot;

        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &r_fd, &slot) == FAILURE) {
                return;
        }

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);
        
        if (f->ca_initialized == 0)
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "device not intialized");
          RETURN_FALSE;            
        }

        if (ca_reset_slot(f, slot) == 0)
        {
          RETURN_TRUE;
        } else {
          RETURN_FALSE;
        }
}

/* {{{ proto bool ca_lock(resource fd)
   Reject coins */
PHP_FUNCTION(ca_lock)
{
        zval     *r_fd;
        php_fd_t *f;
        unsigned char *answer;
        unsigned char request[CA_ANSWER_LENGTH];
        unsigned char result[CA_ANSWER_LENGTH];

        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &r_fd) == FAILURE) {
                return;
        }

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);
        
        if (f->ca_initialized == 0)
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "device not intialized");
          RETURN_FALSE;            
        }
        
        sprintf(request,"%x",CA_REJECT);
        sprintf(result,"%x",CA_REJECT_OK);
        
        ca_write(f->fd, request);
        ca_sleep(CA_DELAY_COINS);
        ca_read(f->fd, answer, CA_ANSWER_LENGTH);
        
        if (ca_hexcmp(answer, result) == 0)
        {
          f->accept_coins = 0;
          RETURN_TRUE;
        } else {
          RETURN_FALSE;
        }
}

/* {{{ proto bool ca_unlock(resource fd)
   Free coin acceptor */
PHP_FUNCTION(ca_unlock)
{
        zval     *r_fd;
        php_fd_t *f;
        unsigned char *answer;
        unsigned char request[CA_ANSWER_LENGTH];
        unsigned char result[CA_ANSWER_LENGTH];
        
        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &r_fd) == FAILURE) {
                return;
        }

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);
        
        if (f->ca_initialized == 0)
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "device not intialized");
          RETURN_FALSE;            
        }
        
        sprintf(request,"%x",CA_FREE);
        sprintf(result,"%x",CA_FREE_OK);

        ca_write(f->fd, request);
        ca_sleep(CA_DELAY_COINS);
        ca_read(f->fd, answer, CA_ANSWER_LENGTH);
                
        if (ca_hexcmp(answer, result) == 0)
        {
          f->accept_coins = 1;
          RETURN_TRUE;
        } else {
          RETURN_FALSE;
        }
}

/* {{{ proto bool ca_accept_coins(resource fd)
   Does coin acceptor accept coins ;-) */
PHP_FUNCTION(ca_accept_coins)
{
        zval     *r_fd;
        php_fd_t *f;

        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &r_fd) == FAILURE) {
                return;
        }

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);
        
        if (f->accept_coins == 1)
        {
          RETURN_TRUE;
        } else {
          RETURN_FALSE;
        }
}


#ifdef HAVE_PROM_SUPPORT	
/* {{{ proto string ca_prom_get(resource fd, string key)
   Get a specified field from prom */
PHP_FUNCTION(ca_prom_get)
{
        zval     *r_fd;
        php_fd_t *f;
        unsigned char	 *field;
        unsigned char	 answer[CA_PROM_ANSWER_LENGTH];
        unsigned char 	 *result;
        long		 field_len;
        int		 rlen, i, diff;
        
        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &r_fd, &field, &field_len) == FAILURE) {
                return;
        }
        
        char request[field_len + 2];

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);       
        
        if (f->ca_initialized == 0)
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "device not intialized");
          RETURN_FALSE;            
        }
        
        if (!is_hex(field, field_len))
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "not a hex string: %s", field);
          RETURN_FALSE;                    
        }
        
        sprintf(request,"%x%s",CA_PROM_HEADER_READ,field);
        
        ca_write(f->fd, request);
        ca_sleep(CA_DELAY_PROM);
        rlen = ca_read(f->fd, answer, CA_PROM_ANSWER_LENGTH);
        
        if (rlen <= 0)
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to read from device: %s", strerror(errno));
          RETURN_FALSE;                           
        }
        
        result = bin2hex(answer, rlen);
        diff = (strlen(result) - (strlen(request) - 2));
        
        unsigned char retval[diff];
        int m;
        
        for (i = (strlen(request) -2); i < strlen(result); i++)
        {
          m = (strlen(request) - 2);
          retval[(i - m)] = result[i];
        }
        
        RETURN_STRINGL(retval, (diff - 2), 1);
}

/* {{{ proto bool ca_prom_set(resource fd, string key, string value[, long write_mode])
   Set a specified key to value */
PHP_FUNCTION(ca_prom_set)
{
        zval     *r_fd;
        php_fd_t *f;
        unsigned char	 *field;
        unsigned char	 answer[CA_PROM_ANSWER_LENGTH];
        unsigned char 	 *result;
        unsigned char	 *value;
        long		 value_len;
        long		 field_len;
        long		 mode;
        int		 rlen, i, diff;
        
        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rss|l", &r_fd, &field, &field_len, &value, &value_len, &mode) == FAILURE) {
                return;
        }
        
        char request[(field_len + 2 + (2 * value_len))];

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);       
        
        if (f->ca_initialized == 0)
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "device not intialized");
          RETURN_FALSE;            
        }
        
        if (!is_hex(field, field_len))
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "not a hex string: %s", field);
          RETURN_FALSE;                    
        }

        php_error_docref(NULL TSRMLS_CC, E_ERROR, "function not yet implemented.");
        RETURN_FALSE;                    
}
#endif

#ifdef HAVE_COMMAND_SUPPORT
/* {{{ proto string ca_command(resource fd, string command[, long delay])
   Get a specified field from prom */
PHP_FUNCTION(ca_command)
{
        if ((COIN_ACCEPTOR_G(command_function) == "Off")
        || (COIN_ACCEPTOR_G(command_function) == "off"))
        {
          php_error_docref(NULL TSRMLS_CC, E_ERROR, "function disabled");
          return;        
        }

        zval     	*r_fd;
        php_fd_t 	*f;
        unsigned char 	*answer;
        char	 	*command;
        int	 	command_len;
        long	 	delay;
        
        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|l", &r_fd, &command, &command_len, &delay) == FAILURE) {
                return;
        }

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);
        
        if (f->ca_initialized == 0)
        {
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "device not intialized");
          RETURN_FALSE;            
        }
        
        if ((!delay) || (delay < 25000) || (delay > 1000000))
        {
          delay = 65535;
        }
        
        ca_write(f->fd, command);
        ca_sleep(delay);
        ca_read(f->fd, answer, CA_ANSWER_LENGTH);
        
        RETURN_STRINGL(answer, strlen(answer), 1);        
}
#endif

/* {{{ proto void ca_close(resource fd)
   Close the descriptor given by fd */
PHP_FUNCTION(ca_close)
{
        zval     	 *r_fd;
        php_fd_t 	 *f;
        unsigned char 	 *answer;
        unsigned char	 request[CA_ANSWER_LENGTH];

        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &r_fd) == FAILURE) {
                return;
        }

        ZEND_FETCH_RESOURCE(f, php_fd_t *, &r_fd, -1, le_coin_acceptor_name, le_coin_acceptor);

        if ((COIN_ACCEPTOR_G(lock_on_close) == "on")
        || (COIN_ACCEPTOR_G(lock_on_close) == "On"))
        {
          sprintf(request, "%x", CA_REJECT);
        
          ca_write(f->fd, request);
          ca_sleep(CA_DELAY_COINS);
          ca_read(f->fd, answer, CA_ANSWER_LENGTH);  
        }
 
        zend_list_delete(Z_LVAL_P(r_fd));
}
/* }}} */
                                                                   

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
